import codecs
import os
import re


class Prepandoc(object):
    def __init__(self, latex_string="", export_file=""):
        self.content = latex_string
        self.export_file = export_file

    def process(self):
        if "\\begin{document}" in self.content:
            self.preamble, self.content = self.content.split("\\begin{document}")
        else:
            self.preamble = "\\documentclass{article}"
            self.content = self.content + "\n\\end{document}"



import re
from latextomd import config


class Postpandoc(object):
    def __init__(self, markdown):
        self.content = markdown

    def process(self):
        self._replace()

        return self.content

    def _replace(self):
        for item in config.postpandoc:
            p = re.compile(item[0])
            self.content = p.sub(item[1], self.content)



import re


# from latextomd import config


class LatexBlocks(object):
    """class LatexBlocks

    Arguments:
        object {string} -- Latex String Source

    Returns:
        string -- Latex String
    """

    def __init__(self, latex_string):
        self.content = latex_string
        self.exercice = 0

    def process(self):
        if "\\begin{exercice}" in self.content:
            self.process_exercice()
        return self.content

    def process_exercice(self):
        """Process block: exercice
        """
        regex_exercice = (
            r"\\begin{exercice}(\[(?P<block_title>.*?)\])?(\[(?P<block_bareme>.*?)\])?"
        )
        self.content = self.content.replace("\\end{exercice}", "\n\n:::\n\n")
        self.lines = self.content.split("\n")
        newlines = []
        for line in self.lines:
            if "\\begin{exercice}" in line:
                matches = re.finditer(regex_exercice, line, re.MULTILINE)
                self.exercice = self.exercice + 1
                for matchNum, match in enumerate(matches, start=1):
                    if match.group(2) and match.group(4):
                        line = f":::exercice Exercice {self.exercice}: {match.group(2)} /{match.group(4)} points\n\n"

                    elif match.group(2) and not match.group(4):
                        line = f":::exercice Exercice {self.exercice}: {match.group(2)}\n\n"
                    elif not match.group(2) and match.group(4):
                        line = f":::exercice Exercice {self.exercice}: /{match.group(4)} points\n\n"
                    else:
                        line = f":::exercice Exercice {self.exercice}:\n\n"

            newlines.append(line)
        self.lines = newlines
        self.content = "\n".join(self.lines)



from latextomd import config
from TexSoup import TexSoup


class Latex(object):
    def __init__(self, latex_string):
        self.soup = TexSoup(latex_string)

    def _delete_commands(self):
        for command in config.del_commands:
            try:
                for include in self.soup.find_all(command):
                    include.delete()
            except ValueError:
                pass

    def _replace_commands(self):
        for command in config.replace_commands:
            print(command[0])
            liste_commands = self.soup.find_all(command[0])
            self.content = str(self.soup)
            for match in liste_commands:
                self.content = self.content.replace(
                    str(match), command[1].replace("S_T_R", match.string)
                )
            self.soup = TexSoup(self.content)

    def process(self):
        self._delete_commands()
        self._replace_commands()
        self.content = str(self.soup)
        math_inline = self.soup.find_all("$")
        for match in math_inline:
            string = str(list(match.descendants)[0])
            self.content = self.content.replace(string, string.strip())

        return self.content



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['latextomd']

package_data = \
{'': ['*']}

install_requires = \
['TexSoup>=0.2.2,<0.3.0', 'chardet>=3.0.4,<4.0.0']

entry_points = \
{'console_scripts': ['latextomd = latextomd.cli:main']}

setup_kwargs = {
    'name': 'latextomd',
    'version': '0.1.5',
    'description': 'Simple project to convert latex in markdown',
    'long_description': '![CI](https://github.com/DavidCouronne/latextomd/workflows/CI/badge.svg)\n\n# latextomd\n\nA simple python package to convert latex to markdown (katex)\n\nDocumentation: https://latextomd.netlify.com/\n\n## Installation\n\n```bash\npip install latextomd\n```\n\n## Basic usage\n\n```bash\nlatextomd -i source.tex -o export.md\n```\n\n# Requirements (Windows 10)\n\n+ miktex\n+ pandoc\n+ perl: [http://strawberryperl.com/](http://strawberryperl.com/)\n\n```{bash}\nchoco install berrybrew\n```\n\n+ Image Magick: [https://imagemagick.org/](https://imagemagick.org/)\n\n```{bash}\nchoco install imagemagick\n```\n\n',
    'author': 'David Couronné',
    'author_email': 'couronne.david@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/DavidCouronne/latextomd',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import codecs
import os
import re

from latextomd import config

from latextomd.postpandoc import Postpandoc
from latextomd.soup import Latex
from latextomd.text_manipulation import LatexString


def _process_preamble(latex_string):
    """Detect Latex Preamble

    Arguments:
        latex_string {string} -- Latex Source

    Returns:
        tupple -- preamble, content
    """
    if r"\begin{document}" in latex_string:
        preamble, content = latex_string.split(r"\begin{document}")
        if r"\end{document}" in content:
            content = content.split(r"\end{document}")[0]
    else:
        preamble = "\\documentclass{article}\n"
        content = latex_string
    return preamble, content


def _strip_lines(latex_string):
    """Strip Lines in Latex Source

    Arguments:
        latex_string {string} -- LateX Source

    Returns:
        string -- Latex with lines striped
    """
    lines = latex_string.splitlines()
    result = []
    for line in lines:
        result.append(line.strip())
    return "\n".join(result)


def _clean_lines(latex_string):
    lines = latex_string.splitlines()
    while lines[0] == "":
        lines = lines[1:]
    while lines[-1] == "":
        lines = lines[:-1]
    content = "\n".join(lines)
    while "\n\n\n" in content:
        content = content.replace("\n\n\n", "\n\n")
    return content


def _delete_blocks(latex_string):
    """delete blocks define in config.del_environnements

    Arguments:
        latex_string {string} -- Latex string

    Returns:
        string -- Latex string
    """
    content = latex_string
    for env in config.del_environnements:
        content = content.replace(env, "")
    return content


def _pandoc(preamble, content):
    """Convert latex to md with pandoc

    Arguments:
        preamble {string} -- Latex preamble: documentclass{...}...
        content {string} -- Latex content

    Returns:
        string -- Markdown
    """
    total = "\n\\begin{document}\n".join([preamble, content]) + "\n\\end{document}"
    f = codecs.open("temp.tex", "w", "utf-8")
    f.write(total)
    f.close()
    # os.system("pandoc temp.tex -o temp.md --katex --from latex --to gfm")
    os.system("pandoc temp.tex -o temp.md")
    with codecs.open("temp.md", "r", "utf-8") as f:
        content = f.read()
    return content
    # os.system(f"dvisvgm temp.dvi")


def to_markdown(latex_string, export_file_name=""):
    content = latex_string
    content = _strip_lines(content)

    preamble, content = _process_preamble(content)
    content = LatexString(content, preamble, export_file_name).process()
    content = _pandoc(preamble, content)
    # content = Latex(content).process()
    content = Postpandoc(content).process()
    content = _delete_blocks(content)
    # content = _strip_lines(content)

    content = _clean_lines(content)
    return content


class LatexToMd(object):
    def __init__(self, latex_string, export_file_name="", options={}):
        """Initialisation LatexToMd

        Arguments:
            latex_string {str} -- latex string

        Keyword Arguments:
            export_file_name {str} -- export file name (default: {""})
        """
        self.content = latex_string
        self.options = options
        self._process_options()

    def _process_options(self):
        try:
            self.pandoc_enumerate = self.options["pandoc_enumerate"]
        except:
            self.pandoc_enumerate = False

    def process(self):
        """Convert Latex String in Markdown String

        Returns:
            {str} -- Markdown String
        """
        self.prepandoc()
        return self.content

    def prepandoc(self):
        self._remove_comments()
        self._replace_simple()
        self._strip_lines()
        self.preamble, self.content = _process_preamble(self.content)
        self.content = _clean_lines(self.content)

    def _remove_comments(self):
        """Remove comments in latex_string
        """
        self.content = re.sub("(?<!\\\\)%.*$", "", self.content, flags=re.M)

    def _replace_simple(self):
        """Replace without regex
        use config.replace_simple
        """
        for replace_simple in config.replace_simple:
            self.content = self.content.replace(replace_simple[0], replace_simple[1])

    def _strip_lines(self):
        """Strip lines in latex string
        """
        lines = self.content.splitlines()
        result = []
        for line in lines:
            result.append(line.strip())
        self.content = "\n".join(result)



import argparse
import codecs
import sys

from latextomd import __version__, latextomd

parser = argparse.ArgumentParser(
    description="""Basic usage: latextomd -i input.tex -o output.md"""
)
parser.add_argument(
    "-v", default=False, help="show version and exit", action="store_true"
)
parser.add_argument("-i", help="input file path. Must be a .tex file")
parser.add_argument(
    "-o", help="output file path (optionnal). By default the input file path with .md"
)
parser.add_argument(
    "--pandoc-enumerate",
    action="store_true",
    help="let pandoc handle enumerate environnements",
)
parser.add_argument("-d", action="store_true", help="debug mode")

args = parser.parse_args()

if args.v:
    print(f"This is latextomd version {__version__}")
    sys.exit(2)


def predict_encoding(file_path, n_lines=20):
    """Predict a file's encoding using chardet"""
    import chardet

    # Open the file as binary data
    with open(file_path, "rb") as f:
        # Join binary lines for specified number of lines
        rawdata = b"".join([f.readline() for _ in range(n_lines)])

    return chardet.detect(rawdata)["encoding"]


def main():
    options = {
        "pandoc_enumerate": args.pandoc_enumerate,
    }

    if not args.i:
        print("An input file must be specified.")
        sys.exit(2)
    source_file = args.i
    if not args.o:
        export_file = source_file.replace(".tex", ".md")
    else:
        export_file = args.o
    print("Source encoding:", predict_encoding(source_file))
    with codecs.open(source_file, "r", "utf-8") as f:
        latex_string = f.read()
        latextomd_object = latextomd.LatexToMd(latex_string, export_file, options)
        markdown_string = latextomd.to_markdown(latex_string, export_file)
        with codecs.open(export_file, "w", "utf-8") as f_out:
            f_out.write(markdown_string)


if __name__ == "__main__":
    main()



import codecs
import os
import re
import subprocess

from latextomd import config
from latextomd.latexblocks import LatexBlocks


class LatexString(object):
    def __init__(self, latex_string, preamble, export_file_name):
        self.content = latex_string
        self.preamble = preamble
        self.nbfigure = 0
        self.export_file_name = export_file_name
        self.figure = []

    def process(self):
        """Process transformation from Latex to markdown

        Returns:
            str -- pre-markdown
        """
        self._remove_comments()
        self._replace_simple()
        self._math_replace()
        self._convertList()
        # self._convertItemize()
        self._convertEnumerate()
        self.content = LatexBlocks(self.content).process()
        self._findTikz()
        self.findPstricks()
        self.replaceFigure()
        return self.content

    def _remove_comments(self):
        self.content = re.sub("(?<!\\\\)%.*$", "", self.content, flags=re.M)

    def _replace_simple(self):
        for replace_simple in config.replace_simple:
            self.content = self.content.replace(replace_simple[0], replace_simple[1])

    def _math_replace(self):
        for item in config.math_sub:
            p = re.compile(item[0])
            self.content = p.sub(item[1], self.content)

    def _convertItemize(self):
        """Agit sur les lignes.
        Converti les environnements itemize en listes html"""
        new_lines = []
        self.lines = self.content.splitlines()

        for line in self.lines:
            if r"\begin{itemize}" in line:
                line = "\n\n"
            elif r"\end{itemize}" in line:
                line = "\n\n"
            elif r"\item" in line:
                line = line.replace(r"\item", "\n\n$\\bullet$ ")
            new_lines.append(line)
        self.lines = new_lines
        self.content = "\n".join(self.lines)

    def _convertList(self):
        """Agit sur les lignes.
        Converti les environnements list en listes html"""
        new_lines = []
        in_list = False
        self.lines = self.content.splitlines()

        for line in self.lines:
            if r"\begin{list}" in line:
                line = "\n\n"
                in_list = True
            elif r"\end{list}" in line:
                line = "\n\n"
                in_list = False
            elif r"\item" in line and in_list:
                line = line.replace(r"\item", "\n\n$\\bullet$ ")
            new_lines.append(line)
        self.lines = new_lines
        self.content = "\n".join(self.lines)

    def _convertEnumerate(self):
        """Agit sur les lignes.
        Converti les environnements enumerate en listes html"""
        level_enumerate = 0
        enumi = 0
        enumii = 0
        new_lines = []
        in_itemize = False
        arabic = "abcdefghijklmnopqrstuvwxz"
        self.lines = self.content.splitlines()

        for line in self.lines:
            if r"\begin{enumerate}" in line or r"\begin{colenumerate}" in line:
                level_enumerate = level_enumerate + 1
                if "start" in line:
                    print(level_enumerate)
                    if level_enumerate == 1:
                        enumi = int(line.split("=")[1].split("]")[0]) - 1
                    else:
                        enumii = int(line.split("=")[1].split("]")[0]) - 1

                line = ""

            elif r"\end{enumerate}" in line or r"\end{colenumerate}" in line:
                if level_enumerate == 2:
                    enumii = 0
                else:
                    enumi = 0
                level_enumerate = level_enumerate - 1
                line = ""
            elif r"\begin{itemize" in line:
                in_itemize = True
            elif r"\end{itemize" in line:
                in_itemize = False
            elif r"\item" in line and level_enumerate != 0 and not in_itemize:
                if level_enumerate == 1:
                    enumi = enumi + 1
                    line = line.replace(r"\item", r"\textbf{" + f"{enumi}. " + "}")
                    line = "\n\n" + line
                else:
                    line = line.replace(
                        r"\item", r"\textbf{" + f"{enumi}.{arabic[enumii]}) " + "}"
                    )
                    enumii = enumii + 1
                    line = "\n\n" + line
            new_lines.append(line)
        self.lines = new_lines
        self.content = "\n".join(self.lines)

    def findPstricks(self):
        """Agit sur les lignes.
        Essaie de trouver les envir
        onnements Pstricks"""
        in_pstricks = False
        lignes_pstricks = []
        pstricks = []
        for line in self.lines:
            if in_pstricks:
                lignes_pstricks.append(line)
                if r"\end{pspicture" in line:
                    in_pstricks = False
                    pstricks.append("\n".join(lignes_pstricks))
                    lignes_pstricks = []
            else:
                if r"\begin{pspicture" in line:
                    in_pstricks = True
                    lignes_pstricks.append(line)
        self.figure = self.figure + pstricks

    def _findTikz(self):
        """Agit sur les lignes.
        Essaie de trouver les envir
        onnements Tikz"""
        self.lines = self.content.splitlines()
        in_tikz = False
        lignes_tikz = []
        tikz = []
        for line in self.lines:
            if in_tikz:
                lignes_tikz.append(line)
                if r"\end{tikz" in line:
                    in_tikz = False
                    tikz.append("\n".join(lignes_tikz))
                    lignes_tikz = []
            else:
                if r"\begin{tikz" in line:
                    in_tikz = True
                    lignes_tikz.append(line)
        self.tikz = tikz
        self.figure = self.figure + self.tikz

    def replaceFigure(self):
        if len(self.figure) == 0:
            return
        total = (
            self.preamble.replace("[dvips]", "")
            + "\n\\begin{document}\n\\thispagestyle{empty}\n"
            + "\n\\newpage\\thispagestyle{empty}\n".join(self.figure)
            + "\n\\newpage\\thispagestyle{empty}\nEMPTY\n\\end{document}"
        )
        if not os.path.exists("temp"):
            os.mkdir("temp")

        f = codecs.open("prepandoc.tex", "w", "utf-8")
        f.write(total)
        f.close()
        os.system("xelatex prepandoc.tex")
        os.system("pdfcrop prepandoc.pdf")
        print(self.export_file_name)
        command = (
            "magick convert -density 600 prepandoc-crop.pdf "
            + self.export_file_name.replace(".md", "")
            + ".jpg"
        )
        os.system(command)
        # os.system("magick convert -density 600 prepandoc-crop.pdf truc-%p.jpg")
        for figure in self.figure:
            self.content = self.content.replace(
                figure,
                "\\includegraphics{./"
                + self.export_file_name.replace(".md", "")
                + "-"
                + str(self.nbfigure)
                + "}\n",
            )
            self.nbfigure = self.nbfigure + 1

        """ os.system("dvisvgm temp")
            try:
                os.rename("temp.svg", "figure"+str(self.nbfigure)+".svg")
            except:
                print("Le fichier figure"+str(self.nbfigure)+".svg existe déjà")
            self.content = self.content.replace(
                figure,
                '![Image](./figure'+str(self.nbfigure)+".svg)") """
